@@ -71,75 +71,6 @@ static int pty_free_memory(char *s, int fd, int argc, char *argv[]) {
     return error;
 }
 
-int invoke_pty_process(
-    char *name, 
-    int argc, char *argv[], 
-    char *slavename, int *masterfd,
-    int *extra_input) {
-
-    char **local_argv = NULL;  /* Local argument vector to pass to GDB */
-    pid_t pid;          /* PID of child process (gdb) to be returned */
-    int i, j = 0, extra = 3;    /* 1 name, 1 NULL */
-    int pin[2] = { -1, -1 };
-
-    if ( !name ) {
-      logger_write_pos ( logger, __FILE__, __LINE__, "name error");
-      return -1;
-    } 
-    
-    /* Create an input pipe to the child program */
-    if ( pipe(pin) == -1 ) {
-        logger_write_pos ( logger, __FILE__, __LINE__, "pipe failed");
-        free_memory(argc, local_argv);
-        return -1;
-    }
-        
-    /* Copy the argv into the local_argv, and NULL terminate it. */
-    local_argv = (char **) xmalloc((argc+extra) * sizeof(char *));
-    local_argv[j++] = xstrdup(name);
-   
-    /* copy in all the data the user entered */ 
-    for (i = 0; i < argc; i++)
-        local_argv[j++] = xstrdup(argv[i]);
-
-    /* Add the read only pipe descriptor */
-    {
-        char num[16];
-        sprintf(num, "%d", pin[0]);
-        local_argv[j++] = xstrdup(num);
-    }
-
-    local_argv[j] = NULL;
-
-    /* Fork into two processes with a shared pty pipe */
-    pid = pty_fork(masterfd, slavename, SLAVE_SIZE, NULL, NULL);
-   
-    if ( pid == -1 ) {          /* error, free memory and return  */
-        logger_write_pos ( logger, __FILE__, __LINE__, "pty_fork failed");
-        pty_free_memory(slavename, *masterfd, argc, local_argv);
-        return -1;
-    } else if ( pid == 0 ) {    /* child */
-        xclose(pin[1]);
-
-        if(execvp(local_argv[0], local_argv) == -1) {
-            logger_write_pos ( logger, __FILE__, __LINE__, "execvp failed");
-            logger_write_pos ( logger, __FILE__, __LINE__, "%s is not in path", name);
-            pty_free_memory(slavename, *masterfd, argc, local_argv);
-            exit(-1);
-        }
-
-        exit(-1);
-    }
-
-    xclose(pin[0]);
-    *extra_input = pin[1];
-
-    if ( pty_free_memory(NULL, -1, argc, local_argv) == -1 )
-        return -1;
-
-    return pid;
-}
-
 int invoke_pty_process_function( 
             char *slavename, int *masterfd, int *extra_input,
             int (*entry)(int)) {
